%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Computation of QNMs for slowly rotating Kerr-de Sitter black holes
%  by means of a quantization condition
%  (c) Semyon Dyatlov, UC Berkeley, 2010 <dyatlov@berkeley.edu>
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% qcAngular.m - compute the angular quantization condition
% KDSD - a KDSData
% omega - the input value of omega, a complex number
% k - the input value of k, an integer
% l - the parameter of the quantization condition (how far away we are),
% a positive integer. We assume that l >= |k|.
% N - the order to which compute the quantization condition, should be at
%   least 1
% lambda - the resulting lambda, a complex number

function lambda = qcAngular(KDSD, omega, k, l, N)

% Set some parameters
KDSD.k = k;
KDSD.omegaR = real(omega);
KDSD.omegaI = imag(omega);
mTemp = l - abs(KDSD.k);

% Maximal depth
dpt = max(2, mTemp + 2 * N - 1);

% Calculate the jet of the functions A and B at zero; see Appendix B.3 of
% the paper for details
A = jetpoly([1, 0, i * (1 - KDSD.alpha), 0, KDSD.alpha], dpt + 2, 0);
jt1 = jetpoly([KDSD.a * KDSD.omegaR - KDSD.k, 0, i * KDSD.a * KDSD.omegaR], dpt + 2, 0);
jt2 = jetpoly([1, 0, i], dpt + 2, 0) * i * KDSD.a * KDSD.omegaI;
B(1, :) = jetmul(jt1, jt1);
B(2, :) = 2 * jetmul(jt1, jt2);
B(3, :) = jetmul(jt2, jt2);
for l = 1 : 3
    B(l, :) = - i * power(1 + KDSD.alpha, 2) * jetmul(jetinv(A), B(l, :));
end

% Invoke qcBarrierTop to get lambda
lambda = i * qcBarrierTop(A, B, mTemp, N);

end