%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Computation of QNMs for slowly rotating Kerr-de Sitter black holes
%  by means of a quantization condition
%  (c) Semyon Dyatlov, UC Berkeley, 2010 <dyatlov@berkeley.edu>
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% qcBarrierTop.m - compute the quantization condition at a barrier top
%   See Appendix B.2 of the paper for details
% A - the jet of the function A at y = 0, to depth at least dpt, see below.
% B - the jet of the function B at y = 0, to depth at least dpt + 2, see below.
%   B(l, :) contains the jet of the function B_{l-1}, in general
% m - the quantization parameter (how deep we need to go)
% N - the degree to which compute the expansion (so, first N + 1 terms)
%   We assume that N > 0.
% lambda - the resulting lambda, a complex number

function lambda = qcBarrierTop(A, B, m, N)

% Maximal depth - see the explanation below
dpt = max(2, m + 2 * N - 1);

% Elements of the asymptotic expansion of lambda
lambdaVal = zeros(1, N + 1);

% lambda0
lambdaVal(1) = B(1, 1);

% U0, jet at zero
U0j = - B(1, 3 : dpt + 2);

% Some more functions: F1 = sqrt(U0 * A), F2 = (y * F1)'
F1j = jetsqrt(jetmul(U0j, A));
F2j = F1j + [0, jetdiff(F1j, 1)];

% Filling in the matrices for Bj and Lj
% The procedure is somewhat inefficient and might need improvement
% Note: all jets are row vectors; therefore, we multiply them by matrices
%   on the right, contrary to the standard convention (for column vectors).
%   Also, the matrix L0mat will be upper triangular.
L0mat = zeros(dpt, dpt);
L1mat = zeros(dpt, dpt - 2);
Bmat = zeros(dpt, dpt, max(size(B, 1), 2));
Id = eye(dpt);
for j = 1 : dpt
    % Apply all the operators to Id(j, :)
    L0mat(j, :) = [0, jetmul(2 * i * F1j, jetdiff(Id(j, :), 1))] ...
        + jetmul(i * F2j, Id(j, :));
    if N > 1
        L1mat(j, :) = jetdiff(jetmul(A, jetdiff(Id(j, :), 1)), 1);
    end
    for l = 1 : size(B, 1)
        Bmat(j, :, l) = jetmul(B(l, :), Id(j, :));
    end
end

% lambda1 and the matrix we will invert all the time
% Zmat is upper triangular and we make Zmat(m + 1, m + 1) = 0
Zmat = L0mat - Bmat(:, :, 2);
lambdaVal(2) = - Zmat(m + 1, m + 1);
Zmat = Zmat + lambdaVal(2) * Id;

% We find an element f of the cokernel of Zmat and normalize it
fvec = zeros(dpt, 1);
fvec(m + 1) = 1;
for j = m : -1 : 1
    fvec(j) = - (Zmat(j, :) * fvec) / Zmat(j, j);
end
fvec = fvec / length(fvec);

% The rest of lambda's and all a's, inductively
% To find lambda_N, we need the (m + 2)-jet of a_{N-2}
% So, we need to know the (m + 2N - 2)-jet of a_0
% Hence the value of dpt above
ajet = zeros(N - 1, dpt);
RHS = zeros(1, dpt);
for j = 1 : N - 1 
    % Filling in a_{j-1}
    for l = 1 : dpt
        if l == m + 1
            if j == 1
                ajet(j, l) = 1 / fvec(m + 1);
            else
                % Might need to change this in case of problems with stability
                ajet(j, l) = 0;
            end
        else
            ajet(j, l) = (RHS(l) - (ajet(j, :) * Zmat(:, l))) / Zmat(l, l);
        end
    end    
    
    % Computing the right-hand side of the next equation
    RHS = - [ajet(j, :) * L1mat, 0, 0];
    for l = 1 : min(size(B, 1) - 2, j) 
        RHS = RHS + ajet(j - l + 1, :) * (Bmat(: , :, l + 2) - lambdaVal(l + 2) * Id);
    end
    
    % Computing lambda_{j+1}
    lambdaVal(j + 2) = RHS * fvec;
    RHS = RHS - lambdaVal(j + 2) * ajet(1, :);
end

% Finally, we sum the first N + 1 terms of the asymptotic series
lambda = sum(lambdaVal);

end