%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Computation of QNMs for slowly rotating Kerr-de Sitter black holes
%  by means of a quantization condition
%  (c) Semyon Dyatlov, UC Berkeley, 2010 <dyatlov@berkeley.edu>
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% qcQNM.m - compute the angular quantization condition
% KDSD - a KDSData
% k - the input value of k, an integer
% l - parameter of the angular quantization condition (how far away we are),
% a positive integer. We assume that l >= |k|.
% m - parameter of the radial quantization condition (how deep the resonance
%   is), a nonnegative integer
% N - the order to which compute the quantization condition, should be at
%   least 1
% eps - the precision in matching the two quantization conditions
% NMax - the maximal number of iterations
% omega - the resulting QNM omega
% lambda - the resulting lambda, a complex number

function [omega, lambda] = qcQNM(KDSD, k, l, m, N, eps, NMax)

% First approximation - formula from Sa Barreto-Zworski
magic = sqrt(1 - 9 * KDSD.Lambda * power(KDSD.M, 2)) / sqrt(27) / KDSD.M;
omega =  magic * ((l + 1/2) - i * (m + 1/2));
omega0 = omega;

cnt = 0;
while 1
    cnt = cnt + 1;
    if cnt > NMax
        error('Too many steps in qcQNM');
    end
    if abs(omega - omega0) > 10 * abs(omega0)
        error('Veered off course in qcQNM');
    end
    
    diff = computeDifference(omega);
    disp(['qcQNM: omega = ', num2str(omega), ', abs(diff) = ', num2str(abs(diff), 2)]);
    if abs(diff) < eps
        break;
    end

    h = 0.001 * magic;
    derR = (computeDifference(omega + h) - computeDifference(omega - h)) / 2 / h;
    derI = (computeDifference(omega + h * i) - computeDifference(omega - h * i)) / 2 / h;
    sol = [real(derR), real(derI) ; imag(derR), imag(derI)] \ [real(diff) ; imag(diff)];
    omega = omega - sol(1) - i * sol(2);
end

lambda = qcRadial(KDSD, omega, k, m, N);

    function ans = computeDifference(omega)
        ans = qcRadial(KDSD, omega, k, m, N) - qcAngular(KDSD, omega, k, l, N);
    end

end