%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Computation of QNMs for slowly rotating Kerr-de Sitter black holes
%  by means of a quantization condition
%  (c) Semyon Dyatlov, UC Berkeley, 2010 <dyatlov@berkeley.edu>
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% qcRadial.m - compute the radial quantization condition
% KDSD - a KDSData
% omega - the input value of omega, a complex number
% k - the input value of k, an integer
% m - the parameter of the quantization condition (how deep the resonance
%   is), a nonnegative integer
% N - the order to which compute the quantization condition, should be at
%   least 1
% lambda - the resulting lambda, a complex number

function lambda = qcRadial(KDSD, omega, k, m, N)

% Set some parameters
KDSD.k = k;
KDSD.omegaR = real(omega);
KDSD.omegaI = imag(omega);

% Maximal depth
dpt = max(2, m + 2 * N - 1);

% Calculate r0
r0 = findr0(KDSD);

% Calculate the jets of A and B at r0; see Appendix B.3 of the paper for
% details
A = jetpoly(KDSD.deltarc, dpt + 2, r0);
jt1 = jetpoly([power(KDSD.a, 2) * KDSD.omegaR - KDSD.a * KDSD.k, 0, KDSD.omegaR], dpt + 2, r0);
jt2 = jetpoly([power(KDSD.a, 2), 0, 1], dpt + 2, r0) * i * KDSD.omegaI;
B(1, :) = jetmul(jt1, jt1);
B(2, :) = 2 * jetmul(jt1, jt2);
B(3, :)= jetmul(jt2, jt2);
for l = 1 : 3
    B(l, :) = - power(1 + KDSD.alpha, 2) * jetmul(B(l, :), jetinv(A));
end

% Invoke qcBarrierTop to compute lambda
lambda = - qcBarrierTop(A, B, m, N);

end